@@ -1,7 +1,7 @@
 /* echo-area.c -- how to read a line in the echo area.
-   $Id: echo-area.c,v 1.10 2007/07/01 21:20:29 karl Exp $
+   $Id: echo-area.c,v 1.14 2008/06/11 09:55:41 gray Exp $
 
-   Copyright (C) 1993, 1997, 1998, 1999, 2001, 2004, 2007
+   Copyright (C) 1993, 1997, 1998, 1999, 2001, 2004, 2007, 2008
    Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
@@ -36,7 +36,7 @@ int info_aborted_echo_area = 0;
 int echo_area_is_active = 0;
 
 /* The address of the last command executed in the echo area. */
-VFunction *ea_last_executed_command = (VFunction *)NULL;
+VFunction *ea_last_executed_command = NULL;
 
 /* Non-zero means that the last command executed while reading input
    killed some text. */
@@ -44,12 +44,12 @@ int echo_area_last_command_was_kill = 0;
 
 /* Variables which hold on to the current state of the input line. */
 static char input_line[1 + EA_MAX_INPUT];
-static char *input_line_prompt;
+static const char *input_line_prompt;
 static int input_line_point;
 static int input_line_beg;
 static int input_line_end;
 static NODE input_line_node = {
-  (char *)NULL, (char *)NULL, (char *)NULL, input_line,
+  NULL, NULL, NULL, input_line,
   EA_MAX_INPUT, 0, N_IsInternal
 };
 
@@ -66,12 +66,12 @@ static int completions_window_p (WINDOW *window);
 /* If non-null, this is a window which was specifically created to display
    possible completions output.  We remember it so we can delete it when
    appropriate. */
-static WINDOW *echo_area_completions_window = (WINDOW *)NULL;
+static WINDOW *echo_area_completions_window = NULL;
 
 /* Variables which keep track of the window which was active prior to
    entering the echo area. */
-static WINDOW *calling_window = (WINDOW *)NULL;
-static NODE *calling_window_node = (NODE *)NULL;
+static WINDOW *calling_window = NULL;
+static NODE *calling_window_node = NULL;
 static long calling_window_point = 0;
 static long calling_window_pagetop = 0;
 
@@ -96,7 +96,7 @@ remember_calling_window (WINDOW *window)
 static void
 restore_calling_window (void)
 {
-  register WINDOW *win, *compwin = (WINDOW *)NULL;
+  register WINDOW *win, *compwin = NULL;
 
   /* If the calling window is still visible, and it is the window that
      we used for completions output, then restore the calling window. */
@@ -110,7 +110,7 @@ restore_calling_window (void)
           window_set_node_of_window (calling_window, calling_window_node);
           calling_window->point = calling_window_point;
           calling_window->pagetop = calling_window_pagetop;
-          compwin = (WINDOW *)NULL;
+          compwin = NULL;
           break;
         }
     }
@@ -144,7 +144,7 @@ restore_calling_window (void)
              calls echo_area_inform_of_deleted_window (), which does the
              right thing. */
 #if defined (UNNECESSARY)
-          echo_area_completions_window = (WINDOW *)NULL;
+          echo_area_completions_window = NULL;
 #endif /* UNNECESSARY */
 
           if (next)
@@ -159,7 +159,7 @@ restore_calling_window (void)
 
 /* Set up a new input line with PROMPT. */
 static void
-initialize_input_line (char *prompt)
+initialize_input_line (const char *prompt)
 {
   input_line_prompt = prompt;
   if (prompt)
@@ -178,7 +178,7 @@ echo_area_after_read (void)
   if (info_aborted_echo_area)
     {
       info_aborted_echo_area = 0;
-      return_value = (char *)NULL;
+      return_value = NULL;
     }
   else
     {
@@ -187,12 +187,12 @@ echo_area_after_read (void)
       else
         {
           int line_len = input_line_end - input_line_beg;
-          return_value = (char *) xmalloc (1 + line_len);
+          return_value = xmalloc (1 + line_len);
           strncpy (return_value, &input_line[input_line_beg], line_len);
           return_value[line_len] = '\0';
         }
     }
-  return (return_value);
+  return return_value;
 }
 
 /* Read a line of text in the echo area.  Return a malloc ()'ed string,
@@ -200,7 +200,7 @@ echo_area_after_read (void)
    active window, so that we can restore it when we need to.  PROMPT, if
    non-null, is a prompt to print before reading the line. */
 char *
-info_read_in_echo_area (WINDOW *window, char *prompt)
+info_read_in_echo_area (WINDOW *window, const char *prompt)
 {
   char *line;
 
@@ -241,7 +241,7 @@ info_read_in_echo_area (WINDOW *window, char *prompt)
     pop_echo_area ();
 
   /* Return the results to the caller. */
-  return (line);
+  return line;
 }
 
 /* (re) Initialize the echo area node. */
@@ -492,6 +492,7 @@ DECLARE_INFO_COMMAND (ea_insert, _("Insert this character"))
   input_line[input_line_point] = key;
   input_line_point++;
   input_line_end++;
+  window_line_map_init (window);
 }
 
 DECLARE_INFO_COMMAND (ea_tab_insert, _("Insert a TAB character"))
@@ -544,7 +545,7 @@ DECLARE_INFO_COMMAND (ea_transpose_chars, _("Transpose characters at point"))
 /*                                                                  */
 /* **************************************************************** */
 
-static char **kill_ring = (char **)NULL;
+static char **kill_ring = NULL;
 static int kill_ring_index = 0; /* Number of kills appearing in KILL_RING. */
 static int kill_ring_slots = 0; /* Number of slots allocated to KILL_RING. */
 static int kill_ring_loc = 0;   /* Location of current yank pointer. */
@@ -559,7 +560,7 @@ DECLARE_INFO_COMMAND (ea_yank, _("Yank back the contents of the last kill"))
 
   if (!kill_ring_index)
     {
-      inform_in_echo_area ((char *) _("Kill ring is empty"));
+      inform_in_echo_area (_("Kill ring is empty"));
       return;
     }
 
@@ -645,6 +646,7 @@ DECLARE_INFO_COMMAND (ea_kill_word, _("Kill the word following the cursor"))
 
       input_line_point = orig_point;
     }
+  window_line_map_init (window);
 }
 
 /* Delete from point to the start of the current word. */
@@ -662,6 +664,7 @@ DECLARE_INFO_COMMAND (ea_backward_kill_word,
       if (input_line_point != orig_point)
         ea_kill_text (orig_point, input_line_point);
     }
+  window_line_map_init (window);
 }
 
 /* The way to kill something.  This appends or prepends to the last
@@ -688,7 +691,7 @@ ea_kill_text (int from, int to)
 
   /* Remember the text that we are about to delete. */
   distance = to - from;
-  killed_text = (char *)xmalloc (1 + distance);
+  killed_text = xmalloc (1 + distance);
   strncpy (killed_text, &input_line[from], distance);
   killed_text[distance] = '\0';
 
@@ -708,7 +711,7 @@ ea_kill_text (int from, int to)
 
       slot = kill_ring_loc;
       old = kill_ring[slot];
-      new = (char *)xmalloc (1 + strlen (old) + strlen (killed_text));
+      new = xmalloc (1 + strlen (old) + strlen (killed_text));
 
       if (killing_backwards)
         {
@@ -737,9 +740,9 @@ ea_kill_text (int from, int to)
         slot = 0;
 
       if (slot + 1 > kill_ring_slots)
-        kill_ring = (char **) xrealloc
-          (kill_ring,
-           (kill_ring_slots += max_retained_kills) * sizeof (char *));
+        kill_ring = xrealloc (kill_ring,
+			      (kill_ring_slots += max_retained_kills)
+			      * sizeof (char *));
 
       if (slot != kill_ring_index)
         free (kill_ring[slot]);
@@ -762,12 +765,12 @@ ea_kill_text (int from, int to)
 /* **************************************************************** */
 
 /* Pointer to an array of REFERENCE to complete over. */
-static REFERENCE **echo_area_completion_items = (REFERENCE **)NULL;
+static REFERENCE **echo_area_completion_items = NULL;
 
 /* Sorted array of REFERENCE * which is the possible completions found in
    the variable echo_area_completion_items.  If there is only one element,
    it is the only possible completion. */
-static REFERENCE **completions_found = (REFERENCE **)NULL;
+static REFERENCE **completions_found = NULL;
 static int completions_found_index = 0;
 static int completions_found_slots = 0;
 
@@ -778,7 +781,7 @@ static REFERENCE *LCD_completion;
 static void build_completions (void), completions_must_be_rebuilt (void);
 
 /* Variable which holds the output of completions. */
-static NODE *possible_completions_output_node = (NODE *)NULL;
+static NODE *possible_completions_output_node = NULL;
 
 static char *compwin_name = "*Completions*";
 
@@ -792,13 +795,13 @@ completions_window_p (WINDOW *window)
       (strcmp (window->node->nodename, compwin_name) == 0))
     result = 1;
 
-  return (result);
+  return result;
 }
 
 /* Workhorse for completion readers.  If FORCE is non-zero, the user cannot
    exit unless the line read completes, or is empty. */
 char *
-info_read_completing_internal (WINDOW *window, char *prompt,
+info_read_completing_internal (WINDOW *window, const char *prompt,
     REFERENCE **completions, int force)
 {
   char *line;
@@ -853,7 +856,7 @@ info_read_completing_internal (WINDOW *window, char *prompt,
           /* If one of the completions matches exactly, then that is okay, so
              return the current line. */
           for (i = 0; i < completions_found_index; i++)
-            if (strcasecmp (completions_found[i]->label, line) == 0)
+            if (mbscasecmp (completions_found[i]->label, line) == 0)
               {
                 free (line);
                 line = xstrdup (completions_found[i]->label);
@@ -864,9 +867,9 @@ info_read_completing_internal (WINDOW *window, char *prompt,
           if (i == completions_found_index)
             {
               if (!completions_found_index)
-                inform_in_echo_area ((char *) _("No completions"));
+                inform_in_echo_area (_("No completions"));
               else
-                inform_in_echo_area ((char *) _("Not complete"));
+                inform_in_echo_area (_("Not complete"));
               continue;
             }
         }
@@ -880,31 +883,31 @@ info_read_completing_internal (WINDOW *window, char *prompt,
   display_cursor_at_point (active_window);
   fflush (stdout);
 
-  echo_area_completion_items = (REFERENCE **)NULL;
+  echo_area_completion_items = NULL;
   completions_must_be_rebuilt ();
 
   /* If there is a previous loop waiting for us, restore it now. */
   if (echo_area_is_active)
     pop_echo_area ();
 
-  return (line);
+  return line;
 }
   
 /* Read a line in the echo area with completion over COMPLETIONS. */
 char *
 info_read_completing_in_echo_area (WINDOW *window,
-    char *prompt, REFERENCE **completions)
+    const char *prompt, REFERENCE **completions)
 {
-  return (info_read_completing_internal (window, prompt, completions, 1));
+  return info_read_completing_internal (window, prompt, completions, 1);
 }
 
 /* Read a line in the echo area allowing completion over COMPLETIONS, but
    not requiring it. */
 char *
 info_read_maybe_completing (WINDOW *window,
-    char *prompt, REFERENCE **completions)
+    const char *prompt, REFERENCE **completions)
 {
-  return (info_read_completing_internal (window, prompt, completions, 0));
+  return info_read_completing_internal (window, prompt, completions, 0);
 }
 
 DECLARE_INFO_COMMAND (ea_possible_completions, _("List possible completions"))
@@ -920,11 +923,11 @@ DECLARE_INFO_COMMAND (ea_possible_completions, _("List possible completions"))
   if (!completions_found_index)
     {
       terminal_ring_bell ();
-      inform_in_echo_area ((char *) _("No completions"));
+      inform_in_echo_area (_("No completions"));
     }
   else if ((completions_found_index == 1) && (key != '?'))
     {
-      inform_in_echo_area ((char *) _("Sole completion"));
+      inform_in_echo_area (_("Sole completion"));
     }
   else
     {
@@ -933,8 +936,8 @@ DECLARE_INFO_COMMAND (ea_possible_completions, _("List possible completions"))
 
       initialize_message_buffer ();
       printf_to_message_buffer (completions_found_index == 1
-                                ? (char *) _("One completion:\n")
-                                : (char *) _("%d completions:\n"),
+                                ? _("One completion:\n")
+                                : _("%d completions:\n"),
 				(void *) (long) completions_found_index,
 				NULL, NULL);
 
@@ -1146,23 +1149,23 @@ DECLARE_INFO_COMMAND (ea_complete, _("Insert completion"))
 
 /* Utility REFERENCE used to store possible LCD. */
 static REFERENCE LCD_reference = {
-    (char *)NULL, (char *)NULL, (char *)NULL, 0, 0, 0
+    NULL, NULL, NULL, 0, 0, 0
 };
 
 static void remove_completion_duplicates (void);
 
 /* Variables which remember the state of the most recent call
    to build_completions (). */
-static char *last_completion_request = (char *)NULL;
-static REFERENCE **last_completion_items = (REFERENCE **)NULL;
+static char *last_completion_request = NULL;
+static REFERENCE **last_completion_items = NULL;
 
 /* How to tell the completion builder to reset internal state. */
 static void
 completions_must_be_rebuilt (void)
 {
   maybe_free (last_completion_request);
-  last_completion_request = (char *)NULL;
-  last_completion_items = (REFERENCE **)NULL;
+  last_completion_request = NULL;
+  last_completion_items = NULL;
 }
 
 /* Build a list of possible completions from echo_area_completion_items,
@@ -1179,14 +1182,14 @@ build_completions (void)
   if (!echo_area_completion_items)
     {
       completions_found_index = 0;
-      LCD_completion = (REFERENCE *)NULL;
+      LCD_completion = NULL;
       return;
     }
 
   /* Check to see if this call to build completions is the same as the last
      call to build completions. */
   len = input_line_end - input_line_beg;
-  request = (char *)xmalloc (1 + len);
+  request = xmalloc (1 + len);
   strncpy (request, &input_line[input_line_beg], len);
   request[len] = '\0';
 
@@ -1204,11 +1207,11 @@ build_completions (void)
 
   /* Always start at the beginning of the list. */
   completions_found_index = 0;
-  LCD_completion = (REFERENCE *)NULL;
+  LCD_completion = NULL;
 
   for (i = 0; (entry = echo_area_completion_items[i]); i++)
     {
-      if (strncasecmp (request, entry->label, len) == 0)
+      if (mbsncasecmp (request, entry->label, len) == 0)
         add_pointer_to_array (entry, completions_found_index,
                               completions_found, completions_found_slots,
                               20, REFERENCE *);
@@ -1216,7 +1219,7 @@ build_completions (void)
       if (!informed_of_lengthy_job && completions_found_index > 100)
         {
           informed_of_lengthy_job = 1;
-          window_message_in_echo_area ((char *) _("Building completions..."),
+          window_message_in_echo_area (_("Building completions..."),
               NULL, NULL);
         }
     }
@@ -1255,7 +1258,7 @@ build_completions (void)
       }
 
     maybe_free (LCD_reference.label);
-    LCD_reference.label = (char *)xmalloc (1 + shortest);
+    LCD_reference.label = xmalloc (1 + shortest);
     /* Since both the sorting done inside remove_completion_duplicates
        and all the comparisons above are case-insensitive, it's
        possible that the completion we are going to return is
@@ -1291,7 +1294,7 @@ compare_references (const void *entry1, const void *entry2)
   REFERENCE **e1 = (REFERENCE **) entry1;
   REFERENCE **e2 = (REFERENCE **) entry2;
 
-  return (strcasecmp ((*e1)->label, (*e2)->label));
+  return mbscasecmp ((*e1)->label, (*e2)->label);
 }
 
 /* Prune duplicate entries from COMPLETIONS_FOUND. */
@@ -1313,14 +1316,14 @@ remove_completion_duplicates (void)
     {
       if (strcmp (completions_found[i]->label,
                   completions_found[i + 1]->label) == 0)
-        completions_found[i] = (REFERENCE *)NULL;
+        completions_found[i] = NULL;
       else
         newlen++;
     }
 
   /* We have marked all the dead slots.  It is faster to copy the live slots
      twice than to prune the dead slots one by one. */
-  temp = (REFERENCE **)xmalloc ((1 + newlen) * sizeof (REFERENCE *));
+  temp = xmalloc ((1 + newlen) * sizeof (REFERENCE *));
   for (i = 0, j = 0; i < completions_found_index; i++)
     if (completions_found[i])
       temp[j++] = completions_found[i];
@@ -1328,7 +1331,7 @@ remove_completion_duplicates (void)
   for (i = 0; i < newlen; i++)
     completions_found[i] = temp[i];
 
-  completions_found[i] = (REFERENCE *)NULL;
+  completions_found[i] = NULL;
   completions_found_index = newlen;
   free (temp);
 }
@@ -1370,7 +1373,7 @@ echo_area_inform_of_deleted_window (WINDOW *window)
   /* If this window was the echo_area_completions_window, then notice that
      the window has been deleted. */
   if (window == echo_area_completions_window)
-    echo_area_completions_window = (WINDOW *)NULL;
+    echo_area_completions_window = NULL;
 }
 
 /* **************************************************************** */
@@ -1382,7 +1385,7 @@ echo_area_inform_of_deleted_window (WINDOW *window)
 /* Push and Pop the echo area. */
 typedef struct {
   char *line;
-  char *prompt;
+  const char *prompt;
   REFERENCE **comp_items;
   int point, beg, end;
   int must_complete;
@@ -1390,7 +1393,7 @@ typedef struct {
   WINDOW *compwin;
 } PUSHED_EA;
 
-static PUSHED_EA **pushed_echo_areas = (PUSHED_EA **)NULL;
+static PUSHED_EA **pushed_echo_areas = NULL;
 static int pushed_echo_areas_index = 0;
 static int pushed_echo_areas_slots = 0;
 
@@ -1400,7 +1403,7 @@ push_echo_area (void)
 {
   PUSHED_EA *pushed;
 
-  pushed = (PUSHED_EA *)xmalloc (sizeof (PUSHED_EA));
+  pushed = xmalloc (sizeof (PUSHED_EA));
   pushed->line = xstrdup (input_line);
   pushed->prompt = input_line_prompt;
   pushed->point = input_line_point;
@@ -1414,7 +1417,7 @@ push_echo_area (void)
   add_pointer_to_array (pushed, pushed_echo_areas_index, pushed_echo_areas,
                         pushed_echo_areas_slots, 4, PUSHED_EA *);
 
-  echo_area_completion_items = (REFERENCE **)NULL;
+  echo_area_completion_items = NULL;
 }
 
 static void
@@ -1447,7 +1450,7 @@ pop_echo_area (void)
 
       /* If the window wasn't found, then it has already been deleted. */
       if (!win)
-        echo_area_completions_window = (WINDOW *)NULL;
+        echo_area_completions_window = NULL;
     }
 
   free (popped);
@@ -1462,9 +1465,9 @@ echo_area_stack_contains_completions_p (void)
 
   for (i = 0; i < pushed_echo_areas_index; i++)
     if (pushed_echo_areas[i]->compwin)
-      return (1);
+      return 1;
 
-  return (0);
+  return 0;
 }
 
 /* **************************************************************** */
@@ -1490,8 +1493,7 @@ pause_or_input (void)
   FD_SET (fileno (stdin), &readfds);
   timer.tv_sec = 2;
   timer.tv_usec = 0;
-  ready = select (fileno (stdin) + 1, &readfds, (fd_set *) NULL,
-                  (fd_set *) NULL, &timer);
+  ready = select (fileno (stdin) + 1, &readfds, NULL, NULL, &timer);
 #endif /* FD_SET */
 }
 
