@@ -1,13 +1,13 @@
 /* indices.c -- deal with an Info file index.
-   $Id: indices.c,v 1.2 2003/02/11 16:39:06 karl Exp $
+   $Id: indices.c,v 1.11 2008/06/11 09:55:42 gray Exp $
 
-   Copyright (C) 1993, 1997, 1998, 1999, 2002, 2003 Free Software
-   Foundation, Inc.
+   Copyright (C) 1993, 1997, 1998, 1999, 2002, 2003, 2004, 2007, 2008
+   Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,10 +15,9 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-   Written by Brian Fox (bfox@ai.mit.edu). */
+   Originally written by Brian Fox (bfox@ai.mit.edu). */
 
 #include "info.h"
 #include "indices.h"
@@ -28,17 +27,17 @@ int show_index_match = 1;
 
 /* In the Info sense, an index is a menu.  This variable holds the last
    parsed index. */
-static REFERENCE **index_index = (REFERENCE **)NULL;
+static REFERENCE **index_index = NULL;
 
 /* The offset of the most recently selected index element. */
 static int index_offset = 0;
 
 /* Variable which holds the last string searched for. */
-static char *index_search = (char *)NULL;
+static char *index_search = NULL;
 
 /* A couple of "globals" describing where the initial index was found. */
-static char *initial_index_filename = (char *)NULL;
-static char *initial_index_nodename = (char *)NULL;
+static char *initial_index_filename = NULL;
+static char *initial_index_nodename = NULL;
 
 /* A structure associating index names with index offset ranges. */
 typedef struct {
@@ -48,22 +47,20 @@ typedef struct {
 } INDEX_NAME_ASSOC;
 
 /* An array associating index nodenames with index offset ranges. */
-static INDEX_NAME_ASSOC **index_nodenames = (INDEX_NAME_ASSOC **)NULL;
+static INDEX_NAME_ASSOC **index_nodenames = NULL;
 static int index_nodenames_index = 0;
 static int index_nodenames_slots = 0;
 
 /* Add the name of NODE, and the range of the associated index elements
    (passed in ARRAY) to index_nodenames. */
 static void
-add_index_to_index_nodenames (array, node)
-     REFERENCE **array;
-     NODE *node;
+add_index_to_index_nodenames (REFERENCE **array, NODE *node)
 {
   register int i, last;
   INDEX_NAME_ASSOC *assoc;
 
   for (last = 0; array[last + 1]; last++);
-  assoc = (INDEX_NAME_ASSOC *)xmalloc (sizeof (INDEX_NAME_ASSOC));
+  assoc = xmalloc (sizeof (INDEX_NAME_ASSOC));
   assoc->name = xstrdup (node->nodename);
 
   if (!index_nodenames_index)
@@ -88,32 +85,30 @@ add_index_to_index_nodenames (array, node)
    indices are concatenated and the result returned.  If WINDOW's info file
    doesn't have any indices, a NULL pointer is returned. */
 REFERENCE **
-info_indices_of_window (window)
-     WINDOW *window;
+info_indices_of_window (WINDOW *window)
 {
   FILE_BUFFER *fb;
 
   fb = file_buffer_of_window (window);
 
-  return (info_indices_of_file_buffer (fb));
+  return info_indices_of_file_buffer (fb);
 }
 
 REFERENCE **
-info_indices_of_file_buffer (file_buffer)
-     FILE_BUFFER *file_buffer;
+info_indices_of_file_buffer (FILE_BUFFER *file_buffer)
 {
   register int i;
-  REFERENCE **result = (REFERENCE **)NULL;
+  REFERENCE **result = NULL;
 
   /* No file buffer, no indices. */
   if (!file_buffer)
-    return ((REFERENCE **)NULL);
+    return NULL;
 
   /* Reset globals describing where the index was found. */
   maybe_free (initial_index_filename);
   maybe_free (initial_index_nodename);
-  initial_index_filename = (char *)NULL;
-  initial_index_nodename = (char *)NULL;
+  initial_index_filename = NULL;
+  initial_index_nodename = NULL;
 
   if (index_nodenames)
     {
@@ -124,7 +119,7 @@ info_indices_of_file_buffer (file_buffer)
         }
 
       index_nodenames_index = 0;
-      index_nodenames[0] = (INDEX_NAME_ASSOC *)NULL;
+      index_nodenames[0] = NULL;
     }
 
   /* Grovel the names of the nodes found in this file. */
@@ -169,7 +164,7 @@ info_indices_of_file_buffer (file_buffer)
     if (!result[i]->filename)
       result[i]->filename = xstrdup (file_buffer->filename);
 
-  return (result);
+  return result;
 }
 
 DECLARE_INFO_COMMAND (info_index_search,
@@ -181,10 +176,7 @@ DECLARE_INFO_COMMAND (info_index_search,
 /* Look up SEARCH_STRING in the index for this file.  If SEARCH_STRING
    is NULL, prompt user for input.  */ 
 void
-do_info_index_search (window, count, search_string)
-     WINDOW *window;
-     int count;
-     char *search_string;
+do_info_index_search (WINDOW *window, int count, char *search_string)
 {
   FILE_BUFFER *fb;
   char *line;
@@ -194,7 +186,7 @@ do_info_index_search (window, count, search_string)
 
   /* The user is selecting a new search string, so flush the old one. */
   maybe_free (index_search);
-  index_search = (char *)NULL;
+  index_search = NULL;
 
   /* If this window's file is not the same as the one that we last built an
      index for, build and remember an index now. */
@@ -203,14 +195,15 @@ do_info_index_search (window, count, search_string)
       (FILENAME_CMP (initial_index_filename, fb->filename) != 0))
     {
       info_free_references (index_index);
-      window_message_in_echo_area (_("Finding index entries..."));
+      window_message_in_echo_area (_("Finding index entries..."),
+          NULL, NULL);
       index_index = info_indices_of_file_buffer (fb);
     }
 
   /* If there is no index, quit now. */
   if (!index_index)
     {
-      info_error (_("No indices found."));
+      info_error (_("No indices found."), NULL, NULL);
       return;
     }
 
@@ -283,9 +276,7 @@ do_info_index_search (window, count, search_string)
 }
 
 int
-index_entry_exists (window, string)
-     WINDOW *window;
-     char *string;
+index_entry_exists (WINDOW *window, char *string)
 {
   register int i;
   FILE_BUFFER *fb;
@@ -340,14 +331,14 @@ DECLARE_INFO_COMMAND (info_next_index_match,
      yet. */
   if (!index_search)
     {
-      info_error (_("No previous index search string."));
+      info_error (_("No previous index search string."), NULL, NULL);
       return;
     }
 
   /* If there is no index, that is an error. */
   if (!index_index)
     {
-      info_error (_("No index entries."));
+      info_error (_("No index entries."), NULL, NULL);
       return;
     }
 
@@ -380,8 +371,8 @@ DECLARE_INFO_COMMAND (info_next_index_match,
   /* If that failed, print an error. */
   if ((i < 0) || (!index_index[i]))
     {
-      info_error (_("No %sindex entries containing \"%s\"."),
-                  index_offset > 0 ? _("more ") : "", index_search);
+      info_error (_("No %sindex entries containing `%s'."),
+                  index_offset > 0 ? (char *) _("more ") : "", index_search);
       return;
     }
 
@@ -410,26 +401,27 @@ DECLARE_INFO_COMMAND (info_next_index_match,
 
     if (partial && show_index_match)
       {
-        int j, ls, start, upper;
+        int k, ls, start, upper;
 
         ls = strlen (index_search);
         start = partial - ls;
         upper = isupper (match[start]) ? 1 : 0;
 
-        for (j = 0; j < ls; j++)
+        for (k = 0; k < ls; k++)
           if (upper)
-            match[j + start] = info_tolower (match[j + start]);
+            match[k + start] = info_tolower (match[k + start]);
           else
-            match[j + start] = info_toupper (match[j + start]);
+            match[k + start] = info_toupper (match[k + start]);
       }
 
     {
       char *format;
 
       format = replace_in_documentation
-        (_("Found \"%s\" in %s. (`\\[next-index-match]' tries to find next.)"));
+        (_("Found `%s' in %s. (`\\[next-index-match]' tries to find next.)"),
+         0);
 
-      window_message_in_echo_area (format, match, name);
+      window_message_in_echo_area (format, match, (char *) name);
     }
 
     free (match);
@@ -447,12 +439,21 @@ DECLARE_INFO_COMMAND (info_next_index_match,
 
   info_set_node_of_window (1, window, node);
 
-  /* Try to find an occurence of LABEL in this node. */
   {
-    long start, loc;
+    long loc;
+    long line = index_index[i]->line_number - 1;
 
-    start = window->line_starts[1] - window->node->contents;
-    loc = info_target_search_node (node, index_index[i]->label, start);
+    if (line >= 0 && line < window->line_count)
+      {
+	/* Jump to the line number specified in the index entry.  */
+	loc = window->line_starts[line] - window->node->contents;
+      }
+    else
+      {
+	/* Try to find an occurence of LABEL in this node. */
+	long start = window->line_starts[1] - window->node->contents;
+	loc = info_target_search_node (node, index_index[i]->label, start);
+      }
 
     if (loc != -1)
       {
@@ -471,13 +472,11 @@ DECLARE_INFO_COMMAND (info_next_index_match,
 /* For every menu item in DIR, search the indices of that file for
    SEARCH_STRING. */
 REFERENCE **
-apropos_in_all_indices (search_string, inform)
-     char *search_string;
-     int inform;
+apropos_in_all_indices (char *search_string, int inform)
 {
   register int i, dir_index;
-  REFERENCE **all_indices = (REFERENCE **)NULL;
-  REFERENCE **dir_menu = (REFERENCE **)NULL;
+  REFERENCE **all_indices = NULL;
+  REFERENCE **dir_menu = NULL;
   NODE *dir_node;
 
   dir_node = info_get_node ("dir", "Top");
@@ -547,7 +546,8 @@ apropos_in_all_indices (search_string, inform)
 	  }
 
         if (this_fb && inform)
-          message_in_echo_area (_("Scanning indices of \"%s\"..."), files_name);
+          message_in_echo_area (_("Scanning indices of `%s'..."),
+              files_name, NULL);
 
         this_index = info_indices_of_file_buffer (this_fb);
         free (this_node);
@@ -573,7 +573,7 @@ apropos_in_all_indices (search_string, inform)
   /* Build a list of the references which contain SEARCH_STRING. */
   if (all_indices)
     {
-      REFERENCE *entry, **apropos_list = (REFERENCE **)NULL;
+      REFERENCE *entry, **apropos_list = NULL;
       int apropos_list_index = 0;
       int apropos_list_slots = 0;
 
@@ -597,24 +597,21 @@ apropos_in_all_indices (search_string, inform)
       free (all_indices);
       all_indices = apropos_list;
     }
-  return (all_indices);
+  return all_indices;
 }
 
 #define APROPOS_NONE \
-   N_("No available info files have \"%s\" in their indices.")
+   N_("No available info files have `%s' in their indices.")
 
 void
-info_apropos (string)
-     char *string;
+info_apropos (char *string)
 {
   REFERENCE **apropos_list;
 
   apropos_list = apropos_in_all_indices (string, 0);
 
   if (!apropos_list)
-    {
-      info_error (_(APROPOS_NONE), string);
-    }
+    info_error (_(APROPOS_NONE), string, NULL);
   else
     {
       register int i;
@@ -654,9 +651,7 @@ DECLARE_INFO_COMMAND (info_index_apropos,
       apropos_list = apropos_in_all_indices (line, 1);
 
       if (!apropos_list)
-        {
-          info_error (_(APROPOS_NONE), line);
-        }
+        info_error (_(APROPOS_NONE), line, NULL);
       else
         {
           register int i;
@@ -664,8 +659,9 @@ DECLARE_INFO_COMMAND (info_index_apropos,
 
           initialize_message_buffer ();
           printf_to_message_buffer
-            (_("\n* Menu: Nodes whoses indices contain \"%s\":\n"), line);
-          line_buffer = (char *)xmalloc (500);
+            (_("\n* Menu: Nodes whose indices contain `%s':\n"),
+             line, NULL, NULL);
+          line_buffer = xmalloc (500);
 
           for (i = 0; apropos_list[i]; i++)
             {
@@ -678,7 +674,7 @@ DECLARE_INFO_COMMAND (info_index_apropos,
               len = pad_to (40, line_buffer);
               sprintf (line_buffer + len, "(%s)%s.",
                        apropos_list[i]->filename, apropos_list[i]->nodename);
-              printf_to_message_buffer ("%s\n", line_buffer);
+              printf_to_message_buffer ("%s\n", line_buffer, NULL, NULL);
             }
           free (line_buffer);
         }
@@ -723,7 +719,7 @@ DECLARE_INFO_COMMAND (info_index_apropos,
 
             old_active = active_window;
             active_window = window;
-            new = window_make_window ((NODE *)NULL);
+            new = window_make_window (NULL);
             active_window = old_active;
           }
 
@@ -745,4 +741,3 @@ DECLARE_INFO_COMMAND (info_index_apropos,
   if (!info_error_was_printed)
     window_clear_echo_area ();
 }
-
