@@ -1,13 +1,13 @@
 /* terminal.c -- how to handle the physical terminal for Info.
-   $Id: terminal.c,v 1.3 2004/04/11 17:56:46 karl Exp $
+   $Id: terminal.c,v 1.7 2008/06/11 09:55:43 gray Exp $
 
    Copyright (C) 1988, 1989, 1990, 1991, 1992, 1993, 1996, 1997, 1998,
-   1999, 2001, 2002, 2004 Free Software Foundation, Inc.
+   1999, 2001, 2002, 2004, 2007, 2008 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,8 +15,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
    Originally written by Brian Fox (bfox@ai.mit.edu). */
 
@@ -52,22 +51,22 @@ extern void tputs ();
    function is called when appropriate instead of its namesake.  Your
    function is called with exactly the same arguments that were passed
    to the namesake function. */
-VFunction *terminal_begin_inverse_hook = (VFunction *)NULL;
-VFunction *terminal_end_inverse_hook = (VFunction *)NULL;
-VFunction *terminal_prep_terminal_hook = (VFunction *)NULL;
-VFunction *terminal_unprep_terminal_hook = (VFunction *)NULL;
-VFunction *terminal_up_line_hook = (VFunction *)NULL;
-VFunction *terminal_down_line_hook = (VFunction *)NULL;
-VFunction *terminal_clear_screen_hook = (VFunction *)NULL;
-VFunction *terminal_clear_to_eol_hook = (VFunction *)NULL;
-VFunction *terminal_get_screen_size_hook = (VFunction *)NULL;
-VFunction *terminal_goto_xy_hook = (VFunction *)NULL;
-VFunction *terminal_initialize_terminal_hook = (VFunction *)NULL;
-VFunction *terminal_new_terminal_hook = (VFunction *)NULL;
-VFunction *terminal_put_text_hook = (VFunction *)NULL;
-VFunction *terminal_ring_bell_hook = (VFunction *)NULL;
-VFunction *terminal_write_chars_hook = (VFunction *)NULL;
-VFunction *terminal_scroll_terminal_hook = (VFunction *)NULL;
+VFunction *terminal_begin_inverse_hook = NULL;
+VFunction *terminal_end_inverse_hook = NULL;
+VFunction *terminal_prep_terminal_hook = NULL;
+VFunction *terminal_unprep_terminal_hook = NULL;
+VFunction *terminal_up_line_hook = NULL;
+VFunction *terminal_down_line_hook = NULL;
+VFunction *terminal_clear_screen_hook = NULL;
+VFunction *terminal_clear_to_eol_hook = NULL;
+VFunction *terminal_get_screen_size_hook = NULL;
+VFunction *terminal_goto_xy_hook = NULL;
+VFunction *terminal_initialize_terminal_hook = NULL;
+VFunction *terminal_new_terminal_hook = NULL;
+VFunction *terminal_put_text_hook = NULL;
+VFunction *terminal_ring_bell_hook = NULL;
+VFunction *terminal_write_chars_hook = NULL;
+VFunction *terminal_scroll_terminal_hook = NULL;
 
 /* **************************************************************** */
 /*                                                                  */
