@@ -1,5 +1,5 @@
 /* defun.c -- @defun and friends.
-   $Id: defun.c,v 1.17 2007/07/01 21:20:32 karl Exp $
+   $Id: defun.c,v 1.18 2007/09/15 23:48:45 karl Exp $
 
    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2007
    Free Software Foundation, Inc.
@@ -399,28 +399,28 @@ defun_internal (enum insertion_type type, int x_p)
     {
     case defun:
     case deftypefun:
-      category = __("Function");
+      category = gdt("Function");
       break;
     case defmac:
-      category = __("Macro");
+      category = gdt("Macro");
       break;
     case defspec:
-      category = __("Special Form");
+      category = gdt("Special Form");
       break;
     case defvar:
     case deftypevar:
-      category = __("Variable");
+      category = gdt("Variable");
       break;
     case defopt:
-      category = __("User Option");
+      category = gdt("User Option");
       break;
     case defivar:
     case deftypeivar:
-      category = __("Instance Variable");
+      category = gdt("Instance Variable");
       break;
     case defmethod:
     case deftypemethod:
-      category = __("Method");
+      category = gdt("Method");
       break;
     default:
       category = next_nonwhite_defun_arg (&scan_args);
@@ -496,24 +496,24 @@ defun_internal (enum insertion_type type, int x_p)
         execute_string (" --- %s: %s %s", category, type_name, defined_name);
         break;
       case defcv:
-        execute_string (" --- %s %s %s: %s", category, __("of"), type_name,
+        execute_string (" --- %s %s %s: %s", category, gdt("of"), type_name,
                         defined_name);
         break;
       case deftypecv:
       case deftypeivar:
-        execute_string (" --- %s %s %s: %s %s", category, __("of"), type_name,
+        execute_string (" --- %s %s %s: %s %s", category, gdt("of"), type_name,
                         type_name2, defined_name);
         break;
       case defop:
-        execute_string (" --- %s %s %s: %s", category, __("on"), type_name,
+        execute_string (" --- %s %s %s: %s", category, gdt("on"), type_name,
                         defined_name);
         break;
       case deftypeop:
-        execute_string (" --- %s %s %s: %s %s", category, __("on"), type_name,
+        execute_string (" --- %s %s %s: %s %s", category, gdt("on"), type_name,
                         type_name2, defined_name);
         break;
       case deftypemethod:
-        execute_string (" --- %s %s %s: %s %s", category, __("on"), type_name,
+        execute_string (" --- %s %s %s: %s %s", category, gdt("on"), type_name,
                         type_name2, defined_name);
         break;
       }
@@ -541,13 +541,13 @@ defun_internal (enum insertion_type type, int x_p)
         case defcv:
         case deftypecv:
         case deftypeivar:
-	  execute_string ("--- %s %s %s: ", category, __("of"), type_name);
+	  execute_string ("--- %s %s %s: ", category, gdt("of"), type_name);
 	  break;
 
         case defop:
         case deftypemethod:
         case deftypeop:
-	  execute_string ("--- %s %s %s: ", category, __("on"), type_name);
+	  execute_string ("--- %s %s %s: ", category, gdt("on"), type_name);
 	  break;
 	} /* switch (base_type)... */
 
@@ -640,13 +640,13 @@ defun_internal (enum insertion_type type, int x_p)
 	execute_string ("@vindex %s\n", defined_name);
 	break;
       case deftypeivar:
-	execute_string ("@vindex %s %s %s\n", defined_name, __("of"),
+	execute_string ("@vindex %s %s %s\n", defined_name, gdt("of"),
                         type_name);
 	break;
       case defop:
       case deftypeop:
       case deftypemethod:
-	execute_string ("@findex %s %s %s\n", defined_name, __("on"),
+	execute_string ("@findex %s %s %s\n", defined_name, gdt("on"),
                         type_name);
 	break;
       case deftp:
