@@ -1,13 +1,13 @@
 /* files.c -- file-related functions for makeinfo.
-   $Id: files.c,v 1.5.2.1 2007/06/27 17:11:00 karl Exp $
+   $Id: files.c,v 1.8 2007/07/01 21:20:32 karl Exp $
 
-   Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2007 Free Software
-   Foundation, Inc.
+   Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2007
+   Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,8 +15,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #include "system.h"
 #include "files.h"
@@ -656,7 +655,7 @@ register_delayed_write (char *delayed_command)
   /* We need the HTML header in the output,
      to get a proper output_position.  */
   if (!executing_string && html)
-    html_output_head ();
+    output_head ();
   /* Get output_position updated.  */
   flush_output ();
 
