@@ -1,8 +1,8 @@
 /* makeinfo.h -- declarations for Makeinfo.
-   $Id: makeinfo.h,v 1.29 2007/07/11 00:16:24 karl Exp $
+   $Id: makeinfo.h,v 1.31 2008/03/26 23:57:12 karl Exp $
 
    Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
-   2005, 2006, 2007 Free Software Foundation, Inc.
+   2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -318,10 +318,10 @@ typedef struct generic_list {
   struct generic_list *next;
 } GENERIC_LIST;
 
-/* Use __ instead of _ to translate strings that end up in the output
-   document.  */
+/* Use `gdt' instead of `_' to translate strings that end up in the
+   output document.  */
 extern char *getdocumenttext (const char *msgid);
-#define __(s) getdocumenttext(s)
+#define gdt(s) getdocumenttext(s)
 
 /* Reverse the order of a list.  */
 extern GENERIC_LIST * reverse_list (GENERIC_LIST *list);
@@ -367,7 +367,7 @@ extern void free_and_clear (char **pointer),
 extern int get_until (char *match, char **string),
   set_paragraph_indent (char *string),
   self_delimiting (int character),
-  search_forward (char *string, int from),
+  search_forward (const char *string, int from),
   search_forward_until_pos (char *string, int from, int end_pos),
   next_nonwhitespace_character (void),
   current_output_column (void),
