@@ -1,12 +1,12 @@
 /* multi.h -- declarations for multi.c.
-   $Id: multi.h,v 1.1.2.1 2007/06/27 17:11:02 karl Exp $
+   $Id: multi.h,v 1.4 2007/07/01 21:20:33 karl Exp $
 
    Copyright (C) 2004, 2007 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -14,9 +14,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-   */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #ifndef MULTI_H
 #define MULTI_H
