@@ -1,8 +1,8 @@
 /* texindex -- sort TeX index dribble output into an actual index.
-   $Id: texindex.c,v 1.22 2007/07/01 21:20:34 karl Exp $
+   $Id: texindex.c,v 1.24 2008/02/22 19:18:25 karl Exp $
 
    Copyright (C) 1987, 1991, 1992, 1996, 1997, 1998, 1999, 2000, 2001,
-   2002, 2003, 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
+   2002, 2003, 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -123,7 +123,7 @@ void *xmalloc (), *xrealloc ();
 static char *concat3 (const char *, const char *, const char *);
 
 int
-main (int argc, char **argv)
+main (int argc, char *argv[])
 {
   int i;
 
@@ -296,7 +296,7 @@ decode_command (int argc, char **argv)
 License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\n\
 This is free software: you are free to change and redistribute it.\n\
 There is NO WARRANTY, to the extent permitted by law.\n"),
-	      "2007");
+	      "2008");
               xexit (0);
             }
           else if ((strcmp (arg, "--keep") == 0) ||
